
local StoneGateV5 = ...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/basic/basicKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Keep materials
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Materials/redRoof", StoneGateSetPrefix .. "_KEEP_RED_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Materials/blueRoof", StoneGateSetPrefix .. "_KEEP_BLUE_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Materials/brownRoof", StoneGateSetPrefix .. "_KEEP_BROWN_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Materials/greyRoof", StoneGateSetPrefix .. "_KEEP_GREY_ROOF_MATERIAL")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Materials/grey2Roof", StoneGateSetPrefix .. "_KEEP_GREY2_ROOF_MATERIAL")

-- Register Keep prefab nodes
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_0_Keep_RootPart", StoneGateSetPrefix .. "_KEEP_ROOT_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_1_Keep_Tiling1Part", StoneGateSetPrefix .. "_KEEP_TILING1_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_2_Keep_Tiling2Part", StoneGateSetPrefix .. "_KEEP_TILING2_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_1_Keep_Tiling1PPart", StoneGateSetPrefix .. "_KEEP_TILING1P_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_2_Keep_Tiling2PPart", StoneGateSetPrefix .. "_KEEP_TILING2P_PREFAB")
StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/step_3_Keep_TopPart", StoneGateSetPrefix .. "_KEEP_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV5:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_KEEP_TOP",
	StoneGateSetPrefix .. "_KEEP_TILING1",
	StoneGateSetPrefix .. "_KEEP_TILING2",
	StoneGateSetPrefix .. "_KEEP_TILING1P",
	StoneGateSetPrefix .. "_KEEP_TILING2P"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the stone gate set main root part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_ROOT_PART",
	Name = StoneGateSetPrefix .. "_KEEP_ROOT_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_ROOT_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_KEEP_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 12,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})

-- Register the stone gate set main root wooden floor part
StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_ROOTP_PART",
	Name = StoneGateSetPrefix .. "_KEEP_ROOTP_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_ROOTP_DESC",
	Category = "CORE",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_KEEP_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING2P_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_KEEP_TILING1P_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 12,
	AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_LORD_MANOR"
})


--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--StoneGateV5:configurePrefabFlagList("models/basic/basicMain.fbx/Prefab/main/step_3_Main_TopPart", { "PLATFORM" })
 
-- Register grass clearing components

StoneGateV5:registerPrefabComponent("models/basic/basicKeep.fbx/Prefab/keep/step_0_Keep_RootPart", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 8 }
})

-- Override building part components properties (generated by the building asset processor) to add building zones
StoneGateV5:registerPrefabComponent("models/basic/basicKeep.fbx/Prefab/keep/step_0_Keep_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 8, 8 }
})

function registerRoofPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/basic/basicKeep.fbx/Prefab/keep/Keep_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5 }
		}
	}
    })

--    StoneGateV5:registerPrefabComponent("models/basic/basicKeep.fbx/Prefab/keep/Keep_" .. _nameFbx .. "Part", {
--	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
--	BuildingZone = { 8, 8 }
--  })
end

registerRoofPart( "RoofA", "ROOF_A" )
registerRoofPart( "RoofB", "ROOF_B" )
registerRoofPart( "RoofC", "ROOF_C" )
registerRoofPart( "RoofD", "ROOF_D" )
registerRoofPart( "RoofE", "ROOF_E" )
registerRoofPart( "RoofF", "ROOF_F" )

---- Ramparts...

-- Apply building asset processor
StoneGateV5:registerAssetProcessor("models/ramparts/rampartKeep.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGateV5:registerAssetId("models/ramparts/rampartKeep.fbx/Prefab/keep/step_4_Keep_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGateV5:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_KEEP_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	}
    })

--    StoneGateV5:registerPrefabComponent("models/ramparts/rampartKeep.fbx/Prefab/keep/step_4_Keep_" .. _nameFbx .. "Part", {
--	DataType = "COMP_BUILDING_PART",
--	BuildingPartType = "MINOR", -- Snaps the decoration only on minor attach nodes (here only on the roofs)
--	BuildingZone = { 4, 4 }
--  })
end

registerRampartPart( "RampartA", "RAMPART_A" )
registerRampartPart( "RampartB", "RAMPART_B" )
registerRampartPart( "RampartC", "RAMPART_C" )
registerRampartPart( "RampartD", "RAMPART_D" )
registerRampartPart( "RampartE", "RAMPART_E" )
registerRampartPart( "RampartF", "RAMPART_F" )
